"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useIonHeaderParallax = void 0;
const React = __importStar(require("react"));
function useIonHeaderParallax({ titleColor, image, expandedColor, maximumHeight = 300, }) {
    const headerRef = React.useRef(null);
    let header;
    let toolbar;
    let toolbarBackground;
    let imageOverlay;
    let colorOverlay;
    let barButtons;
    let scrollContent;
    let headerHeight;
    let headerMinHeight;
    let translateAmt;
    let scaleAmt;
    let scrollTop;
    let lastScrollTop;
    let ticking;
    let originalToolbarBgColor;
    let overlayTitle;
    let ionTitle;
    let overlayButtons;
    let scrollContentPaddingTop;
    React.useEffect(() => {
        setTimeout(() => {
            try {
                initElements();
                initStyles();
                initEvents();
            }
            catch (e) {
                throw e;
            }
        }, 100);
    }, []);
    const initElements = () => {
        if (!(headerRef && headerRef.current))
            throw new ReferenceError('Ref is required to be set to <IonHeader />');
        const parentElement = headerRef.current.parentElement;
        if (!parentElement)
            throw new Error('No parentElemnt');
        header = headerRef.current;
        // check this
        toolbar = header.querySelector('IonToolbar');
        if (!toolbar) {
            throw new Error('Parallax requires a <IonToolbar> or navbar element on the page to work.');
        }
        if (!toolbar.shadowRoot) {
            throw new Error('Parallax requires a shadowRoot <IonToolbar>');
        }
        ionTitle = toolbar.querySelector('IonTitle');
        toolbarBackground = toolbar.shadowRoot.querySelector('.toolbar-background');
        console.log('toolbarBackground', toolbarBackground);
        barButtons = header.querySelector('IonButtons');
        const ionContent = parentElement.querySelector('IonContent');
        if (!ionContent)
            throw new Error('Parallax directive requires an <IonContent> element on the page to work.');
        if (!ionContent.shadowRoot)
            throw new Error('Parallax requires a shadowRoot <ion-content>');
        scrollContent = ionContent.shadowRoot.querySelector('.inner-scroll');
        if (!scrollContent) {
            throw new Error('Parallax directive requires an <IonContent> element on the page to work.');
        }
        headerHeight = scrollContent.clientHeight;
        console.log('headerHeight', headerHeight);
        // Create image overlay
        imageOverlay = document.createElement('div');
        console.log('imageOverlay', imageOverlay);
        imageOverlay.classList.add('image-overlay');
        colorOverlay = document.createElement('div');
        colorOverlay.classList.add('color-overlay');
        colorOverlay.appendChild(imageOverlay);
        header.appendChild(colorOverlay);
        // Copy title and buttons
        overlayTitle = ionTitle && ionTitle.cloneNode(true);
        if (!overlayTitle)
            throw new Error('');
        overlayTitle.classList.add('parallax-title');
        setTimeout(() => {
            var _a;
            const toolbarTitle = (_a = overlayTitle === null || overlayTitle === void 0 ? void 0 : overlayTitle.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector('.toolbar-title');
            toolbarTitle.style.pointerEvents = 'unset';
        });
        if (overlayTitle) {
            imageOverlay.appendChild(overlayTitle);
        }
        if (barButtons) {
            imageOverlay.appendChild(barButtons);
        }
        console.log('finished');
    };
    const initStyles = () => {
        ticking = false;
        if (!scrollContent || !toolbar) {
            return;
        }
        // fetch styles
        headerMinHeight = toolbar.offsetHeight;
        scrollContentPaddingTop = window.getComputedStyle(scrollContent, null).paddingTop.replace('px', '');
        scrollContentPaddingTop = parseFloat(scrollContentPaddingTop);
        if (!toolbarBackground)
            return;
        originalToolbarBgColor = window.getComputedStyle(toolbarBackground, null).backgroundColor;
        if (!originalToolbarBgColor) {
            throw new Error('Error: toolbarBackround is null.');
        }
        // header and title
        header.style.position = 'relative';
        if (overlayTitle) {
            // rerender here
            overlayTitle.style.color = titleColor;
            overlayTitle.style.position = 'absolute';
            overlayTitle.style.width = '100%';
            overlayTitle.style.height = '100%';
            overlayTitle.style.textAlign = 'center';
        }
        // color overlay
        colorOverlay.style.backgroundColor = originalToolbarBgColor;
        colorOverlay.style.height = `${maximumHeight}px`;
        colorOverlay.style.position = 'absolute';
        colorOverlay.style.top = `${-headerMinHeight * 0}px`;
        colorOverlay.style.left = '0';
        colorOverlay.style.width = '100%';
        colorOverlay.style.zIndex = '10';
        colorOverlay.style.pointerEvents = 'none';
        // image overlay
        imageOverlay.style.backgroundColor = expandedColor;
        imageOverlay.style.backgroundImage = `url(${image || ''})`;
        imageOverlay.style.height = '100%';
        imageOverlay.style.width = '100%';
        imageOverlay.style.pointerEvents = 'none';
        imageOverlay.style.backgroundSize = 'cover';
        imageOverlay.style.backgroundPosition = 'center';
    };
    const initEvents = () => { };
    const onScroll = () => {
        if (!scrollContent || !toolbar) {
            return;
        }
        scrollTop = scrollContent.scrollTop;
        if (scrollTop >= 0) {
            translateAmt = scrollTop / 2;
            scaleAmt = 1;
        }
        else {
            translateAmt = 0;
            scaleAmt = -scrollTop / headerHeight + 1;
        }
        // Parallax total progress
        headerMinHeight = toolbar.offsetHeight;
        let progress = (maximumHeight - scrollTop - headerMinHeight) / (maximumHeight - headerMinHeight);
        progress = Math.max(progress, 0);
        // ion-header: set height
        let targetHeight = maximumHeight - scrollTop;
        targetHeight = Math.max(targetHeight, headerMinHeight);
        console.log(lastScrollTop, ticking, overlayButtons, translateAmt, scaleAmt);
    };
    return {
        ref: headerRef,
        onScroll,
    };
}
exports.useIonHeaderParallax = useIonHeaderParallax;
